/************************************************************************/
/* xml2eps.h: main header file						*/
/************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <math.h>
#include <mathc.h>

#define GRX_PSPROCS	"psprocs.ps"	/* PostScript subroutines */
#define GRX_PSFONTS	"encode.ps"	/* list of reencoded fonts */

#define GRX_MAXBUF	256		/* read buffer size */
#define GRX_PATHLENGTH	256		/* file path length */
#define GRX_RLEN	160		/* length of a ressource string */
#define GRX_ALEN	24		/* length of an attribute name */
#define GRX_NATT	25		/* number of attributes per tag */
#define GRX_ATTLEN	((GRX_RLEN + GRX_ALEN + 4) * GRX_NATT)
#define GRX_MAXDATA	500		/* number of points in a curve */
#define GRX_MSTACK	10		/* size of math stack */
#define GRX_NPAR	20		/* number of function parameters */

#define GRX_GOOD	0		/* no error */
#define GRX_EOF		1		/* premature file end */
#define GRX_UNKNOWN_TAG	2
#define GRX_ENDTAG	3		/* end tag out of order */
#define GRX_FAIL	4		/* wrong invocation */
#define GRX_NOMEM	5		/* no RAM or scratch file allocation */
#define GRX_NOT_XML	6		/* input file of wrong structure */

#define GRX_LINEAR	0		/* computation of curve lengths */
#define GRX_CIRCULAR	1

#define XML		0		/* interpretation modes */
#define PS		1
#define COMMENT		2

enum line_patterns { 			/* line patterns */
  SOLID	= 0,
  DASH,
  DOT,
  DASH_DOT,
  DASH_DOT_DOT,
  LDASH,
  LDASH_DASH,
  LDASH_DASH_DASH
};


/* structures */

struct tag_handle {			/* XML tag reference */
  char *name;					/* XML tag name */
  int (*process)();				/* XML-->EPS translation */
  struct attlist *global;			/* global attribute list */
  const char **attnames;			/* attribute name list */
};

struct attlist {			/* tag attribute list */
  char att[GRX_NATT][GRX_RLEN];
};

struct objaux {				/* for parameter fitting */
  double *par;
  char *funct;					/* RPN function descriptor */
  int *ifit;					/* indices of adj. params */
  double *x, *y;				/* data array */
  int m, n;
};



/* prototypes */

/* parsing, unit conversion */
int identify(const char *s, const char *slist[]);
int parse_xml(FILE *xml, int tag, const struct tag_handle *tagset);
int parse_attlist(const char *attstring, const char *attnames[],
struct attlist *a);
int parse_global(FILE *style, const struct tag_handle *tagset);
double psunits(const char *s);
int recognize(const char *tag, const struct tag_handle *tagset,
char **attr);

/* character translation */
char *utf2ps2(const char *c);
char *utf2ps3(const char *c);
void print_special(const char *c);

/* graphics auxiliary */
void curve_length(int n, double *x, double *y, int mode, double *t);
int function(double x, double o, const double *par, const char *op, double *y);
int math_op(double *x, int n, const double *xh, const double *par,
const char *op);
int transform_xy(double x, double y, const double *par, const char *xop,
const char *yop, double *xt, double *yt);
int optimize(const double *x, const double *y, int n, double *par,
const char *rpn, const char *fit);
int objfct(double *adjpar, double *r, struct objaux *aux);
void print_text(const char *s, double x, double y, const char *xalign,
const char *yalign, const char *fontfamily, const char *fontweight,
const char *fontshape, const char *fontsize, double phi);
void scale(double smin, double smax, double *delta1, int *major);
int set_color(const char *color);
int set_font(const char *family, const char *weight, const char *shape,
int size);
int set_line_style(double width, int pattern);
void split_unit(const char *s, double *v, char *u);

/* tag translations - general */
int x_dummy(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_diagram(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_area(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_arrow(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_bold(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_bolditalic(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_comment(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_curve(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_eps(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_italic(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_line(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_marker(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_parameters(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_roman(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_sub(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_super(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_symbol(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_text(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_xaxis(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
int x_yaxis(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a);
