/************************************************************************/
/* cartesian: XML tag interpretation procedures for Cartesian diagrams	*/
/************************************************************************/
#include "../include/xml2eps.h"


/* global variables */

extern int interpretation_mode, actfontsize, actfont, radix_style;
extern double par[GRX_NPAR];
extern const char *diagram_names[], *area_names[], *arrow_names[],
  *curve_names[], *eps_names[], *line_names[], *marker_names[],
  *text_names[], *xaxis_names[], *yaxis_names[],
  *families[], *fonts[], *patterns[], *markers[];
extern struct attlist diagram;
extern FILE *xml;


/*************************************************************12.04.2002*/
/* !DOCTYPE								*/
/************************************************************************/
int x_dummy(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  return (GRX_GOOD);
}

/*************************************************************21.03.2003*/
/* diagram								*/
/************************************************************************/
int x_diagram(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int i, err, xbox, ybox;
  char *diagr_llx, *diagr_lly, *diagr_urx, *diagr_ury, *graph_llx, *graph_lly,
    *graph_urx, *graph_ury, *scale_llx, *scale_lly, *scale_urx, *scale_ury,
    *radix, aux[GRX_PATHLENGTH], record[256];
  double g0, g1, h0, h1;
  FILE *psprocs;

  diagr_llx = (char *) a->att[1];	/* location of diagram on paper */
  diagr_lly = (char *) a->att[2];
  diagr_urx = (char *) a->att[3];
  diagr_ury = (char *) a->att[4];
  graph_llx = (char *) a->att[5];	/* location of scaled area in ru */
  graph_lly = (char *) a->att[6];
  graph_urx = (char *) a->att[7];
  graph_ury = (char *) a->att[8];
  scale_llx = (char *) a->att[9];	/* range of user coordinates */
  scale_lly = (char *) a->att[10];
  scale_urx = (char *) a->att[11];
  scale_ury = (char *) a->att[12];
  radix = (char *) a->att[13];		/* decimal radix symbol */

  radix_style = (!strcmp(radix, "comma")) ? 1 : 0;
  i = sscanf(diagr_llx, "%*lf%s", aux);
  err = (i < 1 || !strcmp(aux, "ru"));
  i = sscanf(diagr_lly, "%*lf%s", aux);
  err = err || (i < 1 || !strcmp(aux, "ru"));
  i = sscanf(diagr_urx, "%*lf%s", aux);
  err = err || (i < 1 || !strcmp(aux, "ru"));
  i = sscanf(diagr_ury, "%*lf%s", aux);
  err = err || (i < 1 || !strcmp(aux, "ru"));
  if (err) {
    fprintf(stderr, "error: <diagram> graph frame corners specified in "
    "relative units\n");
    return (err);
  }
  printf("%s\n", "%!PS-Adobe-2.0 EPSF-2.0");
  printf("%s\n", "%%Title: Cartesian diagram");
  printf("%s\n", "%%Creator: Grafix-3.0 by ukd");
  printf("%s %d %d %d %d\n", "%%BoundingBox: ", (int) psunits(diagr_llx),
  (int) psunits(diagr_lly), (int) psunits(diagr_urx), (int) psunits(diagr_ury));
  printf("%s\n", "%%Pages: 1");
  printf("%s\n", "%%DocumentFonts: Symbol Helvetica Helvetica-Oblique");
  printf("%s\n", "%%+ Helvetica-Bold Helvetica-BoldOblique");
  printf("%s\n", "%%+ Courier Courier-Oblique");
  printf("%s\n", "%%+ Courier-Bold Courier-BoldOblique");
  printf("%s\n", "%%+ Times-Bold Times-BoldItalic Times-Roman Times-Italic");
  printf("%s\n", "%%EndComments");
  printf("%s\n", "%!");

  /* units and transformations:
     1 ru (relative unit) corresponds to the width of the diagram.
     1 xu is the user-defined unit of the abscissa.
     1 yu is the user-defined unit of the ordinate. */

  printf("\n%s\n", "% BEGIN PREAMBLE (definitions of macros, fonts, etc.)");
  printf("/mm {%g mul} def\n", 72.0/25.4);
  printf("/cm {%g mul} def\n", 72.0/2.54);
  printf("/mm {%g mul} def\n", 72.0);
  xbox =  (int) (psunits(diagr_urx) - psunits(diagr_llx));
  ybox =  (int) (psunits(diagr_ury) - psunits(diagr_lly));
  printf("/ru {%d mul} def\n", xbox);
  sscanf(graph_llx, "%lf", &g0);
  sscanf(graph_urx, "%lf", &g1);
  sscanf(scale_llx, "%lf", &h0);
  sscanf(scale_urx, "%lf", &h1);
  printf("/xu {%g sub %g div %g add %d mul} def\n", h0, (h1 - h0) / (g1 - g0),
  g0, xbox);
  sscanf(graph_lly, "%lf", &g0);
  sscanf(graph_ury, "%lf", &g1);
  sscanf(scale_lly, "%lf", &h0);
  sscanf(scale_ury, "%lf", &h1);
  printf("/yu {%g sub %g div %g add %d mul} def\n", h0, (h1 - h0) / (g1 - g0),
  g0, xbox);

  sprintf(aux, "%s/ps/%s", GRX_HOME, GRX_PSPROCS);
  psprocs = fopen(aux, "r");
  if (psprocs == NULL) {
    fprintf(stderr, "error: <diagram> cannot find or open %s\n", GRX_PSPROCS);
    return (GRX_FAIL);
  }
  while (fgets(record, 256, psprocs) != NULL) fputs(record, stdout);
  fclose(psprocs);

  sprintf(aux, "%s/ps/%s", GRX_HOME, GRX_PSFONTS);
  psprocs = fopen(aux, "r");
  if (psprocs == NULL) {
    fprintf(stderr, "error: <diagram> cannot find or open %s\n", GRX_PSFONTS);
    return (GRX_FAIL);
  }
  while (fgets(record, 256, psprocs) != NULL) fputs(record, stdout);
  fclose(psprocs);

  printf("/ScaledArea {\n");		/* define max. clipping path */
  printf("  newpath\n");
  sscanf(scale_llx, "%lf", &g0);
  sscanf(scale_lly, "%lf", &h0);
  printf("    %g xu %g yu m\n", g0, h0);
  sscanf(scale_urx, "%lf", &g1);
  printf("    %g xu %g yu l\n", g1, h0);
  sscanf(scale_ury, "%lf", &h1);
  printf("    %g xu %g yu l\n", g1, h1);
  printf("    %g xu %g yu l\n", g0, h1);
  printf("  closepath\n");
  printf("} def\n");
  printf("\n%s\n", "% END PREAMBLE");
  printf("%d %d translate\n", (int) psunits(diagr_llx),
  (int) psunits(diagr_lly));

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

#if 0
  printf("newpath\n");			/* frame diagram */
  printf("  0 0 moveto\n");
  printf("  %d 0 lineto\n", xbox);
  printf("  %d %d lineto\n", xbox, ybox);
  printf("  0 %d lineto\n", ybox);
  printf("closepath stroke\n");
#endif

  printf("showpage\n");
  printf("%s\n", "%%Trailer");
  printf("%s\n", "%%Pages: 1");
  printf("%s\n", "%%EndTrailer");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_area								*/
/************************************************************************/
int x_area(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int i, n, err, m;
  double x[GRX_MAXDATA], y[GRX_MAXDATA], t[GRX_MAXDATA], b[GRX_MAXDATA],
    c[GRX_MAXDATA], d[GRX_MAXDATA], bh[GRX_MAXDATA], ch[GRX_MAXDATA],
    dh[GRX_MAXDATA], xh, yh, dx, dy, xmin, xmax, ymin, ymax;
  char *width, *pattern, *interpolation, *linecolor, *fillcolor,
    buffer[GRX_MAXDATA * GRX_ALEN], *p, xs[GRX_RLEN], ys[GRX_RLEN], unit[3];
  struct attlist loc;

  loc = *a;
  width = loc.att[0];
  pattern = loc.att[1];
  interpolation = loc.att[2];
  linecolor = loc.att[3];
  fillcolor = loc.att[4];
  i = parse_attlist(attributes, area_names, &loc);

  fscanf(xml, "%[^<]", buffer);		/* no protection against overflow! */
  n = 0;
  p = buffer;
  while (*p != '\0') {
    i = sscanf(p, "%s %s%n", xs, ys, &m);
    p += m;
    if (i >= 2) {
      split_unit(xs, &x[n], unit);
      x[n] = psunits(xs);
      if (!strcmp(unit, "xu")) x[n] += psunits(diagram.att[5])
      - psunits(diagram.att[9]);
      split_unit(ys, &y[n], unit);
      y[n] = psunits(ys);
      if (!strcmp(unit, "yu")) y[n] += psunits(diagram.att[6])
      - psunits(diagram.att[10]);
      if (n < GRX_MAXDATA - 2) n++;
    }
  }
  if (n == GRX_MAXDATA)
  fprintf(stderr, "warning: <area> buffer overflow, some vertices skipped\n");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf("/Area {\n");
  err = 1;
  printf("  newpath\n");
  printf("    %g %g m \n", x[0], y[0]);
  if (strcmp(interpolation, "none") || n < 3) {
    for (i = 1, t[0] = 0.0; i < n; i++)
      t[i] = t[i - 1] + sqrt(SQ(x[i] - x[i - 1]) + SQ(y[i] - y[i - 1]));
    t[n] = t[n - 1] + sqrt(SQ(x[0] - x[n - 1]) + SQ(y[0] - y[n - 1]));
    x[n] = x[0];
    y[n] = y[0];
    if (!strcmp(interpolation, "spline")) {
      err = ipl_pspline(n+1, t, x, b, c, d);
      err = ipl_pspline(n+1, t, y, bh, ch, dh);
    }
    else {
      err = ipl_akima(n, t, x, b, c, d);
      err = ipl_akima(n, t, y, bh, ch, dh);
    }
    if (!err) {
      dx = 0.005 * t[n];
      xh = dx;
      while (xh < t[n]) {
	printf("    %g %g l\n", ipl_peval(xh, n + 1, t, x, b, c, d, 0),
	ipl_peval(xh, n + 1, t, y, bh, ch, dh, 0));
	xh += dx;
      }
    }
    else
      fprintf(stderr, "warning: <area> spline interpolation failed with "
      "error %d\n", err);
  }
  else 
    for (i = 1; i < n; i++) printf("    %g %g l \n", x[i], y[i]);
  printf("  closepath\n");
  printf("} def\n");
  if ((err = set_color(fillcolor)) != 0) 
    fprintf(stderr, "warning: <area fillcolor=\"%s\"> not permissible - "
    "skipping\n", fillcolor);
  else
    printf("Area fill\n");
  if ((err = set_color(linecolor)) != 0) 
    fprintf(stderr, "warning: <area linecolor=\"%s\"> not permissible - "
    "skipping\n", linecolor);
  if (strcmp(pattern, "blank")) {
    if ((err = set_line_style(psunits(width), identify(pattern, patterns)))
    != 0) {
      if (err)
	fprintf(stderr, "warning: <area width=\"%s\"> not permissible - "
	"using default\n", width);
      else
	fprintf(stderr, "warning: <area pattern=\"%s\"> not permissible - "
	"using default\n", pattern);
    }
    printf("Area stroke\n");
  }
  return GRX_GOOD;
}

/************************************************************************/
/* x_arrow								*/
/************************************************************************/
int x_arrow(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err;
  double v, phi;
  char *xpoint, *ypoint, *xtail, *ytail, *width, *pattern, *pointangle,
    *pointsize, *color, u[3];
  struct attlist loc;

  loc = *a;
  xpoint = loc.att[0];
  ypoint = loc.att[1];
  xtail = loc.att[2];
  ytail = loc.att[3];
  width = loc.att[4];
  pattern = loc.att[5];
  pointangle = loc.att[6];
  pointsize = loc.att[7];
  color = loc.att[8];
  err = parse_attlist(attributes, arrow_names, &loc);

  printf("\ngsave\n");
  if ((err = set_line_style(psunits(width), identify(pattern, patterns)))
  != 0) {
    if (err)
      fprintf(stderr, "warning: <arrow width=\"%s\"> not permissible - "
      "using default\n", width);
    else
      fprintf(stderr, "warning: <arrow pattern=\"%s\"> not permissible - "
      "using default\n", pattern);
  }
  if ((err = set_color(color)) != 0) 
    fprintf(stderr, "warning: <arrow color=\"%s\"> not permissible - "
    "using default\n", color);
  printf("  newpath\n");
  split_unit(xpoint, &v, u);
  printf("\n    %g %s ", v, u);
  split_unit(ypoint, &v, u);
  printf("%g %s ", v, u);
  split_unit(xtail, &v, u);
  printf("%g %s ", v, u);
  split_unit(ytail, &v, u);
  printf("%g %s ", v, u);
  sscanf(pointangle, "%lf", &phi);
  split_unit(pointsize, &v, u);
  printf("%g %g %s Arrow\n", phi, v, u);
  printf("  stroke\n");
  printf("grestore\n");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_comment								*/
/************************************************************************/
int x_comment(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err;

  interpretation_mode = COMMENT;

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  interpretation_mode = XML;
  return (GRX_GOOD);
}

/************************************************************************/
/* x_curve								*/
/************************************************************************/
int x_curve(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int i, n, err, m, cols;
  double *x, *y, *t, *b, *c, *d, *bh, *ch, *dh, *col3, xh, yh, dx, dy,
    h, xy[3], xt, yt;
  char *width, *pattern, *interpolation, *mode, *symbol, *version, *size,
    *xmathops, *ymathops, *color, *funct, *fit, *rimwidth, *psmarker,
    *openstyle, *error, *proc, *clip;
  struct attlist loc;
  FILE *fp, *buffer;
  const size_t size_lf = sizeof(double);

  loc = *a;
  width = loc.att[0];
  pattern = loc.att[1];
  interpolation = loc.att[2];
  mode = loc.att[3];
  symbol = loc.att[4];
  version = loc.att[5];
  size = loc.att[6];
  xmathops = loc.att[7];
  ymathops = loc.att[8];
  color = loc.att[9];
  funct = loc.att[10];
  fit = loc.att[11];
  rimwidth = loc.att[12],
  error = loc.att[13];
  proc = loc.att[14];
  clip = loc.att[15];
  i = parse_attlist(attributes, curve_names, &loc);

  if ((buffer = tmpfile()) == NULL) {
    fprintf(stderr, "error: <curve> could not open scratch file\n");
    return (GRX_NOMEM);
  }
  n = 0;
  cols = (!strcmp(error, "none")) ? 2 : 3;
  while (1) {
    if (cols == 2) {
      if (fscanf(xml, "%lf%lf", &xy[0], &xy[1]) != 2) break;
    }
    else {
      if (fscanf(xml, "%lf%lf%lf", &xy[0], &xy[1], &xy[2]) != 3) break;
    }
    n++;
    fwrite(xy, sizeof(double), cols, buffer);
  }
  rewind(buffer);
#ifdef USE_VALLOC			/* use old (pre-POSIX) method */
  /* valloc() has been declared obsolete, but may still be necessary
     on some platforms */
  if ((x = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (y = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || cols == 3
  && (col3 = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (b = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (c = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (d = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (t = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (bh = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (ch = (double *) valloc((size_t) (sizeof(double) * n))) == NULL
  || (dh = (double *) valloc((size_t) (sizeof(double) * n))) == NULL) {
    fprintf(stderr, "error: <curve> memory allocation problem\n");
    return (GRX_NOMEM);
  }
#else					/* POSIX-style method */
  if (posix_memalign((void **) &x, size_lf, size_lf * n)
  || posix_memalign((void **) &y, size_lf, size_lf * n)
  || cols == 3 && posix_memalign((void **) &col3, size_lf, size_lf * n)
  || posix_memalign((void **) &b, size_lf, size_lf * n)
  || posix_memalign((void **) &c, size_lf, size_lf * n)
  || posix_memalign((void **) &d, size_lf, size_lf * n)
  || posix_memalign((void **) &t, size_lf, size_lf * n)
  || posix_memalign((void **) &bh, size_lf, size_lf * n)
  || posix_memalign((void **) &ch, size_lf, size_lf * n)
  || posix_memalign((void **) &dh, size_lf, size_lf * n)) {
    fprintf(stderr, "error: <curve> memory allocation problem\n");
    return (GRX_NOMEM);
  }
#endif
  for (i = 0; i < n; i++) {
    fread(xy, sizeof(double), cols, buffer);
    x[i] = xy[0];
    y[i] = xy[1];
    if (cols == 3) col3[i] = xy[2];
  }
  fclose(buffer);

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

#if 0			/* better do this after interpolation */
  if ((err = math_op(x, n, par, xmathops)) != 0) fprintf(stderr,
  "warning: <curve> transformation of x values failed with error %d\n", err);
  if ((err = math_op(y, n, par, ymathops)) != 0) fprintf(stderr,
  "warning: <curve> transformation of y values failed with error %d\n", err);
#endif

  printf("\ngsave\n");
  if (!strcmp(clip, "diagram"))
    printf("  ScaledArea clip\n");
  else if (!strcmp(clip, "area"))
    printf("  Area clip\n");

  printf("  2 setlinejoin\n");

  if ((err = set_color(color)) != 0) 
    fprintf(stderr, "warning: <curve color=\"%s\"> not permissible - "
    "using default\n", color);

  if (strcmp(pattern, "blank")) {
    if ((err = set_line_style(psunits(width), identify(pattern, patterns)))
    != 0) {
      if (err)
	fprintf(stderr, "warning: <curve width=\"%s\"> not permissible - "
	"using default\n", width);
      else
	fprintf(stderr, "warning: <curve pattern=\"%s\"> not permissible - "
	"using default\n", pattern);
    }

    err = 1;
    printf("  newpath\n");
    if (strcmp(interpolation, "none")) {
      if (!strcmp(mode, "x")) {
	if (!strcmp(interpolation, "spline")) {
	  if ((err = ipl_spline(n, x, y, b, c, d)) == 0) {
	    i = (!strcmp(proc, "diff1")) ? 1 : 0;
	    if (!transform_xy(x[0], ipl_eval(x[0], n, x, y, b, c, d, i),
	    par, xmathops, ymathops, &xt, &yt))
	    printf("    %g xu %g yu m\n", xt, yt);
	    dx = 0.005 * (x[n - 1] - x[0]);
	    xh = x[0] + dx;
	    while (xh < x[n - 1] + 0.1 * dx) {
	      if (!transform_xy(xh, ipl_eval(xh, n, x, y, b, c, d, i),
	      par, xmathops, ymathops, &xt, &yt))
	      printf("    %g xu %g yu l\n", xt, yt);
	      xh += dx;
	    }
	  }
	  else
	    fprintf(stderr, "warning: <curve> interpolation failed with "
	    "error %d\n", err);
	}
	else if (!strcmp(interpolation, "akima")) {
	  if ((err = ipl_akima(n, x, y, b, c, d)) == 0) {
	    i = (!strcmp(proc, "diff1")) ? 1 : 0;
	    if (!transform_xy(x[0], ipl_eval(x[0], n, x, y, b, c, d, i),
	    par, xmathops, ymathops, &xt, &yt))
	    printf("    %g xu %g yu m\n", xt, yt);
	    dx = 0.005 * (x[n - 1] - x[0]);
	    xh = x[0] + dx;
	    while (xh < x[n - 1] + 0.1 * dx) {
	      if (!transform_xy(xh, ipl_eval(xh, n, x, y, b, c, d, i),
	      par, xmathops, ymathops, &xt, &yt))
	      printf("    %g xu %g yu l\n", xt, yt);
	      xh += dx;
	    }
	  }
	  else
	    fprintf(stderr, "warning: <curve> interpolation failed with "
	    "error %d\n", err);
	}
	else {
	  if (*fit != '\0') {		/* optimize parameters */
	    if ((err = optimize(x, y, n, par, funct, fit)) != GRX_GOOD)
	      fprintf(stderr, "warning: <curve> optimization failed with "
	      "error %d\n", err);
	    else {
	      for (i = 0; i < n; i++) fprintf(stderr, "p%d = %15.7e\n",
	      i, x[i]);
	    }
	  }
	  if ((err = function(x[0], 0.0, par, funct, &yh)) == GRX_GOOD
	  && x[n - 1] != x[0]) {
	    if (!transform_xy(x[0], yh, par, xmathops, ymathops, &xt, &yt))
	    printf("    %g xu %g yu m\n", xt, yt);
	    dx = 0.005 * (x[n - 1] - x[0]);
	    xh = x[0] + dx;
	    while ((xh - x[n - 1]) / dx < 0.1) {
	      if ((err = function(xh, 0.0, par, funct, &yh)) != 0) break;
	      if (!transform_xy(xh, yh, par, xmathops, ymathops, &xt, &yt))
	      printf("    %g xu %g yu l\n", xt, yt);
	      xh += dx;
	    }
	  }
	  if (err) fprintf(stderr, "warning: function evaluation failed "
	  "with error %d\n", err);
	}
      }
      else if (!strcmp(mode, "y")) {
	if (!strcmp(interpolation, "spline"))
	  err = ipl_spline(n, y, x, b, c, d);
	else
	  err = ipl_akima(n, y, x, b, c, d);
	if (!err) {
	  i = (!strcmp(proc, "diff1")) ? 1 : 0;
	  if (!transform_xy(ipl_eval(y[0], n, y, x, b, c, d, i), y[0],
	  par, xmathops, ymathops, &xt, &yt))
	  printf("    %g xu %g yu m\n", xt, yt);
	  dy = 0.005 * (y[n - 1] - y[0]);
	  yh = y[0] + dy;
	  while (yh < y[n - 1] + 0.1 * dy) {
	    if (!transform_xy(ipl_eval(yh, n, y, x, b, c, d, i), yh,
	    par, xmathops, ymathops, &xt, &yt))
	    printf("    %g xu %g yu l\n", xt, yt);
	    yh += dy;
	  }
	}
	else
	  fprintf(stderr, "warning: <curve> interpolation failed with "
	  "error %d\n", err);
      }
      else {				/* parametric interpolation */
#if 0
	h = psunits("1yu") / psunits("1xu");
	for (i = 1, t[0] = 0.0; i < n; i++) t[i] = t[i - 1]
	+ sqrt(SQ(h * (x[i] - x[i - 1])) + SQ(y[i] - y[i - 1]));
#endif
        curve_length(n, x, y, GRX_CIRCULAR, t);
	if (!strcmp(interpolation, "spline")) {
	  err = ipl_spline(n, t, x, b, c, d);
	  err = ipl_spline(n, t, y, bh, ch, dh);
	}
	else {
	  err = ipl_akima(n, t, x, b, c, d);
	  err = ipl_akima(n, t, y, bh, ch, dh);
	}
	if (!err) {
	  if (!transform_xy(ipl_eval(t[0], n, t, x, b, c, d, 0),
	    ipl_eval(t[0], n, t, y, bh, ch, dh, 0),
	    par, xmathops, ymathops, &xt, &yt))
	    printf("    %g xu %g yu m\n", xt, yt);
	  dx = 0.005 * t[n - 1];
	  xh = dx;
	  while (xh < t[n - 1] + 0.1 * dx) {
	    if (!transform_xy(ipl_eval(xh, n, t, x, b, c, d, 0),
	    ipl_eval(xh, n, t, y, bh, ch, dh, 0),
	    par, xmathops, ymathops, &xt, &yt))
	    printf("    %g xu %g yu l\n", xt, yt);
	    xh += dx;
	  }
	}
	else
	  fprintf(stderr, "warning: <curve> interpolation failed with "
	  "error %d\n", err);
      }
    }
    else {
        if (!transform_xy(x[0], y[0], par, xmathops, ymathops, &xt, &yt))
	printf("    %g xu %g yu m \n", xt, yt);
      for (i = 1; i < n; i++) {
        if (!transform_xy(x[i], y[i], par, xmathops, ymathops, &xt, &yt))
	printf("    %g xu %g yu l \n", xt, yt);
      }
    }
    printf("  stroke\n");
    set_line_style(psunits(width), identify("solid", patterns));
  }

  if (!strcmp(error, "dy")) {
    sscanf(rimwidth, "%lf", &h);
    h = MIN(1.0, MAX(0.0, h));
    for (i = 0; i < n; i++) {
      if (!transform_xy(x[i], y[i] - col3[i], par, xmathops, ymathops,
      &xt, &yt)
      && !transform_xy(x[i], y[i] + col3[i], par, xmathops, ymathops,
      &xt, &yh))
      printf("  %f %f %g xu %g yu %g yu Yerror\n", h, psunits(size), xt,
      yt, yh);
    }
  }
  printf("  0 setlinejoin\n");

  if ((m = identify(symbol, markers)) >= 0) {
    openstyle = (strcmp(version, "filled") == 0) ? "false" : "true";
    sscanf(rimwidth, "%lf", &h);
    h = MIN(1.0, MAX(0.0, h));
    for (i = 0; i < n; i++) {
      if (!transform_xy(x[i], y[i], par, xmathops, ymathops, &xt, &yt))
      printf("  %s %f %f %g xu %g yu %s\n", openstyle,
      h, psunits(size), xt, yt, markers[m]);
    }
  }
  printf("grestore\n");

  free(x);
  free(y);
  free(b);
  free(c);
  free(d);
  free(t);
  free(bh);
  free(ch);
  free(dh);
  return (GRX_GOOD);
}

/************************************************************************/
/* x_eps								*/
/************************************************************************/
int x_eps(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int c, err;
  double v;
  char *file, *x, *y, *xscale, *yscale, u[3];
  struct attlist loc;
  FILE *fp;

  loc = *a;
  file = loc.att[0];
  x = loc.att[1];
  y = loc.att[2];
  xscale = loc.att[3];
  yscale = loc.att[4];
  err = parse_attlist(attributes, eps_names, &loc);

  printf("\nBeginEPS\n");
  if ((fp = fopen(file, "r")) == NULL) {
    fprintf(stderr, "error: cannot open file %s\n", file);
    return GRX_FAIL;
  }
  split_unit(x, &v, u);
  printf("%g %s ", v, u);
  split_unit(y, &v, u);
  printf("%g %s translate\n", v, u);
  split_unit(xscale, &v, u);
  printf("%g ", v);
  split_unit(yscale, &v, u);
  printf("%g scale\n", v);
  while ((c = fgetc(fp)) != EOF) putchar(c);
  fclose(fp);
  printf("\nEndEPS\n");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_line								*/
/************************************************************************/
int x_line(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err;
  double v;
  char *x0, *y0, *x1, *y1, *width, *pattern, *color, u[3];
  struct attlist loc;

  loc = *a;
  x0 = loc.att[0];
  y0 = loc.att[1];
  x1 = loc.att[2];
  y1 = loc.att[3];
  width = loc.att[4];
  pattern = loc.att[5];
  color = loc.att[6];
  err = parse_attlist(attributes, line_names, &loc);

  printf("\ngsave\n");
  if ((err = set_line_style(psunits(width), identify(pattern, patterns)))
  != 0) {
    if (err == 1)
      fprintf(stderr, "warning: <line width=\"%s\"> not permissible - "
      "using default\n", width);
    else
      fprintf(stderr, "warning: <line pattern=\"%s\"> not permissible - "
      "using default\n", pattern);
  }
  if ((err = set_color(color)) != 0) 
    fprintf(stderr, "warning: <line color=\"%s\"> not permissible - "
    "using default\n", color);
  printf("  newpath\n");
  split_unit(x0, &v, u);
  printf("    %g %s ", v, u);
  split_unit(y0, &v, u);
  printf("%g %s m\n", v, u);
  split_unit(x1, &v, u);
  printf("    %g %s ", v, u);
  split_unit(y1, &v, u);
  printf("%g %s l\n", v, u);
  printf("  stroke\n");
  printf("grestore\n");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_marker								*/
/************************************************************************/
int x_marker(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int i, err;
  double v, h;
  char *x, *y, *symbol, *version, *size, *color, *rimwidth, u[3];
  struct attlist loc;

  loc = *a;
  x = loc.att[0];
  y = loc.att[1];
  symbol = loc.att[2];
  version = loc.att[3];
  size = loc.att[4];
  color = loc.att[5];
  rimwidth = loc.att[6];
  err = parse_attlist(attributes, marker_names, &loc);

  printf("\ngsave\n");
  if ((err = set_color(color)) != 0) 
    fprintf(stderr, "warning: <marker color=\"%s\"> not permissible - "
    "using default\n", color);
  if ((i = identify(symbol, markers)) <= 0) {
    fprintf(stderr, "warning: <marker symbol=\"%s\"> not permissible - "
    "using default\n", symbol);
    i = 0;
  }
  printf("  %s ", ((!strcmp(version, "filled")) ? "false" : "true"));
  sscanf(rimwidth, "%lf", &h);
  h = MIN(1.0, MAX(0.0, h));
  printf("%f ", h);
  split_unit(size, &v, u);
  printf("%g %s ", v, u);
  split_unit(x, &v, u);
  printf("%g %s ", v, u);
  split_unit(y, &v, u);
  printf("%g %s ", v, u);
  printf("%s\n", markers[i]);
  printf("grestore\n");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_parameters								*/
/************************************************************************/
int x_parameters(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int i, n, err;

  n = -1;
  do {
    if (n < GRX_NPAR - 1) n++;
    i = fscanf(xml, "%lf", &par[n]);
  } while (i == 1);
  if (n == GRX_NPAR - 1) {
    fprintf(stderr, "warning: <parameters> parameter buffer overflow \n");
    return (GRX_FAIL);
  }

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  return (GRX_GOOD);
}

/************************************************************************/
/* x_text								*/
/************************************************************************/
int x_text(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, i;
  double xval, yval, phi;
  char *x, *y, *xalign, *yalign, *fontfamily, *fontweight, *fontshape,
    *fontsize, *angle, *color, xunit[3], yunit[3];
  struct attlist loc;

  loc = *a;
  x = loc.att[0];			/* coordinates of text anchor point */
  y = loc.att[1];
  xalign = loc.att[2];
  yalign = loc.att[3];
  fontfamily = loc.att[4];
  fontweight = loc.att[5];
  fontshape = loc.att[6];
  fontsize = loc.att[7];
  angle = loc.att[8];
  color = loc.att[9];
  i = parse_attlist(attributes, text_names, &loc);

  printf("\ngsave\n");
  if ((err = set_color(color)) != 0) 
    fprintf(stderr, "warning: <text color=\"%s\"> not permissible - "
    "using default\n", color);
  split_unit(x, &xval, xunit);		/* move to text location */
  split_unit(y, &yval, yunit);
  printf("  %g %s %g %s translate 0 0 m\n", xval, xunit, yval, yunit);
  sscanf(angle, "%lf", &phi);
  if (phi != 0.0) printf("  %g rotate\n", phi);
  printf("  /MixedText\n");		/* set up a PS procedure for text */
  printf("  { /final exch def\n");
  set_font(fontfamily, fontweight, fontshape, (int) psunits(fontsize));
  printf("    (");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  printf("  } def\n");			/* end of PS procedure */
  printf("  false MixedText\n");	/* construct text outline */
  printf("  flattenpath pathbbox\n");	/* stack: llx lly urx ury */
  printf("  exch 4 -1 roll sub ");	/* stack: lly ury width */
  if (!strcmp(xalign, "right"))		/* now do the x alignment */
    printf("neg\n");
  else if (!strcmp(xalign, "center"))
    printf("-0.5 mul\n");
  else
    printf("pop 0\n");
  printf("3 1 roll ");			/* stack: dx lly ury */
  if (!strcmp(yalign, "ceiling"))	/* now do y alignment */
    printf("  pop pop %g ", -1.2 * psunits(fontsize));
  else if (!strcmp(yalign, "cap"))
    printf("  pop pop /%s findfont /FontBBox get 3 get %f mul neg ",
    fonts[actfont], 0.001 * actfontsize);
  else if (!strcmp(yalign, "halfcap"))
    printf("  pop pop /%s findfont /FontBBox get 3 get %f mul neg ",
    fonts[actfont], 0.0005 * actfontsize);
  else if (!strcmp(yalign, "floor"))
    printf("  pop pop /%s findfont /FontBBox get 1 get %f mul neg ",
    fonts[actfont], 0.001 * actfontsize);
  else if (!strcmp(yalign, "top"))
    printf("  currentpoint exch pop sub neg exch pop ");
  else if (!strcmp(yalign, "center"))
    printf("  currentpoint exch pop sub -0.5 mul exch pop ");
  else if (!strcmp(yalign, "bottom"))
    printf("  currentpoint 4 1 roll pop pop sub ");
  else
    printf("  pop pop 0 ");
  printf("0 0 m M true MixedText\n");	/* print the text */
  printf("grestore\n");
  actfont = -1;				/* grestore might unset the font */
  return (GRX_GOOD);
}

/************************************************************************/
/* x_xaxis								*/
/************************************************************************/
int x_xaxis(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, i, npower, m;
  double xmin, xmax, x, dxminor, dxmajor, power, w, yval, ymin, ymax,
    tval, ycorr, xomit, xlast, xmin0, xmax0;
  char *y, *majorincr, *minorincr, *numbering, *fontfamily, *fontsize,
    *ticsize, *omit, *linewidth, aux[GRX_RLEN], *yalign, tunit[3];
  struct attlist loc;

  loc = *a;
  y = loc.att[0];
  majorincr = loc.att[1];
  minorincr = loc.att[2];
  numbering = loc.att[3];
  fontfamily = loc.att[4];
  fontsize = loc.att[5];
  ticsize = loc.att[6];
  omit = loc.att[7];
  linewidth = loc.att[8];
  i = parse_attlist(attributes, xaxis_names, &loc);

  sscanf(diagram.att[10], "%lf", &ymin);
  sscanf(diagram.att[12], "%lf", &ymax);
  if (!strcmp(y, "ymin"))
    yval = ymin;
  else if (!strcmp(y, "ymax"))
    yval = ymax;
  else
    sscanf(y, "%lf", &yval);

  yalign = (!strcmp(numbering, "above")) ? "floor" : "ceiling";

  ycorr = psunits(ticsize) / (psunits(diagram.att[4])
  - psunits(diagram.att[2])) * (ymax - ymin);	/* approx. tic size in yu */
  if (yval > ymin && !strcmp(numbering, "below"))
    ycorr *= -2.0;
  else if (yval < ymax && !strcmp(numbering, "above"))
    ycorr *= 2.0;
  else
    ycorr = 0.0;

  sscanf(diagram.att[9], "%lf", &xmin);
  sscanf(diagram.att[11], "%lf", &xmax);
  if (!strcmp(majorincr, "auto") || !strcmp(minorincr, "auto")) {
    scale(xmin, xmax, &dxminor, &m);
    dxmajor = m * dxminor;
  }
  else {
    sscanf(majorincr, "%lf", &dxmajor);
    sscanf(minorincr, "%lf", &dxminor);
    m = (int) (dxmajor / dxminor + 0.5);
  }

  set_line_style(psunits(linewidth), SOLID);
  printf("\nnewpath\n");		/* draw axis */
  printf("%g xu %g yu m\n", xmin, yval);
  printf("%g xu %g yu l\n", xmax, yval);
  printf("stroke\n");

  xmin0 = xmin;
  xmin = floor(xmin / dxmajor) * dxmajor;
  xmax0 = xmax;
  xmax = ceil(xmax / dxmajor) * dxmajor;

  if (sscanf(omit, "%lf%n", &xomit, &i) < 1)
    xomit = DHIGH;
  else
    omit += i;

  if (strcmp(numbering, "none")) { 
    x = (fabs(xmin) > fabs(xmax)) ? fabs(xmin) : fabs(xmax);
    npower = floor(log10(x));
    if (npower >= -2 && npower <= 3) npower = 0;
    power = pow(10.0, (double) npower);
    xlast = (npower == 0) ? xmax + 0.1 * dxminor : xmax - 0.5 * dxminor;
    for (x = xmin; x < xlast; x += dxmajor) {
      if (x < xmin0 || x > xmax0) continue;
      if (fabs(x - xomit) < dxminor) {
	if (sscanf(omit, ",%lf%n", &xomit, &i) < 1)
	  xomit = DHIGH;
	else
	  omit += i;
	continue;
      }
      w = floor(x / dxmajor + 0.5) * dxmajor / power;
      setlocale(LC_NUMERIC, (radix_style == 0) ? "C" : "de_DE");
      if (w < 0)
	sprintf(aux, "\\261%g", fabs(w));
      else
	sprintf(aux, "%g", w);
      setlocale(LC_NUMERIC, "C");
      print_text(aux, x, yval + ycorr, "center", yalign, fontfamily,
      "medium", "roman", fontsize, 0.0);
    }
    if (npower != 0) {			/* write exponent */
      if (npower < 0)
	sprintf(aux, "E\\261%d", -npower);
      else
	sprintf(aux, "E%+d", npower);
      print_text(aux, xmax, yval + ycorr, "right", yalign, fontfamily,
      "medium", "roman", fontsize, 0.0);
    }
  }

  split_unit(ticsize, &tval, tunit);	/* draw tic marks */
  printf("\ngsave\n");
  printf("newpath\n");
  printf("%g xu %g yu m\n", xmin, ymin);
  printf("%g xu %g yu l\n", xmax, ymin);
  printf("%g xu %g yu l\n", xmax, ymax);
  printf("%g xu %g yu l\n", xmin, ymax);
  printf("closepath clip\n");
  printf("newpath\n");
  x = xmin;
  i = 0;
  while (x <= xmax0) {
    if (x >= xmin0) {
      printf("%g xu %g yu m\n", x, yval);
      if (i % m == 0)
        printf("0 %g %s M 0 %g %s L\n", 2 * tval, tunit, -4 * tval, tunit);
      else
        printf("0 %g %s M 0 %g %s L\n", tval, tunit, -2 * tval, tunit);
    }
    x += dxminor;
    i++;
  }
  printf("stroke\n");
  printf("grestore\n");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_yaxis								*/
/************************************************************************/
int x_yaxis(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, i, npower, m;
  double ymin, ymax, y, dyminor, dymajor, power, w, xval, xmin, xmax,
    tval, xcorr, yomit, ylast, ymin0, ymax0;
  char *x, *majorincr, *minorincr, *numbering, *fontfamily, *fontsize,
    *ticsize, *omit, *linewidth, aux[GRX_RLEN], *xalign, tunit[3];
  struct attlist loc;

  loc = *a;
  x = loc.att[0];
  majorincr = loc.att[1];
  minorincr = loc.att[2];
  numbering = loc.att[3];
  fontfamily = loc.att[4];
  fontsize = loc.att[5];
  ticsize = loc.att[6];
  omit = loc.att[7];
  linewidth = loc.att[8];
  i = parse_attlist(attributes, yaxis_names, &loc);

  sscanf(diagram.att[9], "%lf", &xmin);
  sscanf(diagram.att[11], "%lf", &xmax);
  if (!strcmp(x, "xmin"))
    xval = xmin;
  else if (!strcmp(x, "xmax"))
    xval = xmax;
  else
    sscanf(x, "%lf", &xval);

  xalign = (!strcmp(numbering, "left")) ? "right" : "left";

  xcorr = 0.6 * psunits(fontsize);
  if (xval > xmin && xval < xmax) xcorr += 2.0 * psunits(ticsize);
  if (!strcmp(numbering, "left")) xcorr = -xcorr;
  xcorr *= (xmax - xmin) / (psunits(diagram.att[3]) - psunits(diagram.att[1]));

  sscanf(diagram.att[10], "%lf", &ymin);
  sscanf(diagram.att[12], "%lf", &ymax);
  if (!strcmp(majorincr, "auto") || !strcmp(minorincr, "auto")) {
    scale(ymin, ymax, &dyminor, &m);
    dymajor = m * dyminor;
  }
  else {
    sscanf(majorincr, "%lf", &dymajor);
    sscanf(minorincr, "%lf", &dyminor);
    m = (int) (dymajor/dyminor + 0.5);
  }

  set_line_style(psunits(linewidth), SOLID);
  printf("\nnewpath\n");		/* draw axis */
  printf("%g xu %g yu m\n", xval, ymin);
  printf("%g xu %g yu l\n", xval, ymax);
  printf("stroke\n");

  ymin0 = ymin;
  ymin = floor(ymin / dymajor + 0.1) * dymajor;
  ymax0 = ymax;
  ymax = ceil(ymax / dymajor - 0.1) * dymajor;

  if (sscanf(omit, "%lf%n", &yomit, &i) < 1)
    yomit = DHIGH;
  else
    omit += i;

  if (strcmp(numbering, "none")) { 
    y = (fabs(ymin) > fabs(ymax)) ? fabs(ymin) : fabs(ymax);
    npower = floor(log10(y));
    if (npower >= -1 && npower <= 3) npower = 0;
    power = pow(10.0, (double) npower);
    ylast = (npower == 0) ? ymax + 0.1 * dyminor : ymax - 0.5 * dyminor;
    for (y = ymin; y < ylast; y += dymajor) {
      if (y < ymin0 || y > ymax0) continue;
      if (fabs(y - yomit) < dyminor) {
	if (sscanf(omit, ",%lf%n", &yomit, &i) < 1)
	  yomit = DHIGH;
	else
	  omit += i;
	continue;
      }
      w = floor(y / dymajor + 0.5) * dymajor / power;
      setlocale(LC_NUMERIC, (radix_style == 0) ? "C" : "de_DE");
      if (w < 0)
	sprintf(aux, "\\261%g", fabs(w));
      else
	sprintf(aux, "%g", w);
      setlocale(LC_NUMERIC, "C");
      print_text(aux, xval + xcorr, y, xalign, "center", fontfamily,
      "medium", "roman", fontsize, 0.0);
    }
    if (npower != 0) {			/* write exponent */
      if (npower < 0)
	sprintf(aux, "E\\261%d", -npower);
      else
	sprintf(aux, "E%+d", npower);
      print_text(aux, xval + xcorr, y, xalign, "top", fontfamily,
      "medium", "roman", fontsize, 0.0);
    }
  }

  split_unit(ticsize, &tval, tunit);	/* draw tic marks */
  printf("\ngsave\n");
  printf("newpath\n");
  printf("%g xu %g yu m\n", xmin, ymin);
  printf("%g xu %g yu l\n", xmax, ymin);
  printf("%g xu %g yu l\n", xmax, ymax);
  printf("%g xu %g yu l\n", xmin, ymax);
  printf("closepath clip\n");
  printf("newpath\n");
  y = ymin;
  i = 0;
  while (y <= ymax0) {
    if (y >= ymin0) {
      printf("%g xu %g yu m\n", xval, y);
      if (i%m == 0)
        printf("%g %s 0 M %g %s 0 L\n", 2 * tval, tunit, -4 * tval, tunit);
      else
        printf("%g %s 0 M %g %s 0 L\n", tval, tunit, -2 * tval, tunit);
    }
    y += dyminor;
    i++;
  }
  printf("stroke\n");
  printf("grestore\n");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_bold								*/
/************************************************************************/
int x_bold(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfont;

  printf(") final { show } { false charpath } ifelse\n");
  oldactfont = actfont;
  actfont = actfont - actfont % 4 + 2;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfont = oldactfont;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_bolditalic								*/
/************************************************************************/
int x_bolditalic(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfont;

  printf(") final { show } { false charpath } ifelse\n");
  oldactfont = actfont;
  actfont = actfont - actfont % 4 + 3;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfont = oldactfont;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_italic								*/
/************************************************************************/
int x_italic(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfont;

  printf(") final { show } { false charpath } ifelse\n");
  oldactfont = actfont;
  actfont = actfont - actfont % 4 + 1;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfont = oldactfont;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_roman								*/
/************************************************************************/
int x_roman(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfont;

  printf(") final { show } { false charpath } ifelse\n");
  oldactfont = actfont;
  actfont = actfont - actfont % 4;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfont = oldactfont;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_symbol								*/
/************************************************************************/
int x_symbol(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfont;

  printf(") final { show } { false charpath } ifelse\n");
  oldactfont = actfont;
  actfont = identify("Symbol", fonts);
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfont = oldactfont;
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_super								*/
/************************************************************************/
int x_super(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfontsize;

  printf(") final { show } { false charpath } ifelse\n");
  printf("0 %f M\n", 0.35 * ((double) actfontsize));
  oldactfontsize = actfontsize;
  actfontsize = (int) (actfontsize * 0.7 + 0.5);
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfontsize = oldactfontsize;
  printf("currentpoint m 0 %f M\n", -0.35 * ((double) actfontsize));
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}

/************************************************************************/
/* x_sub								*/
/************************************************************************/
int x_sub(FILE *xml, int tag_id, const struct tag_handle *tagset,
const char *attributes, const struct attlist *a) {
  int err, oldactfontsize;

  printf(") final { show } { false charpath } ifelse\n");
  printf("0 %f M\n", -0.35 * ((double) actfontsize));
  oldactfontsize = actfontsize;
  actfontsize = (int) (actfontsize * 0.7 + 0.5);
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");

  if ((err = parse_xml(xml, tag_id, tagset)) != 0) return (err);

  printf(") final { show } { false charpath } ifelse\n");
  actfontsize = oldactfontsize;
  printf("currentpoint m 0 %f M\n", 0.35 * ((double) actfontsize));
  printf("/%s findfont %d scalefont setfont\n", fonts[actfont], actfontsize);
  printf("(");
  return (GRX_GOOD);
}
