/*************************************************************06.03.2014*/
/* colorset: compute a set of RGB colors spanning a given hue range	*/
/*									*/
/* ref.: https://de.wikipedia.org/wiki/HSV-Farbraum			*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin:
     1.	initial hue angle in degrees (0° = red).
     2. final hue angle (240° = pure blue, 300° = magenta).
     3. brightness (range 0 to 1).
     4. number of colors to be generated.				*/
/************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

int main(void) {
  int i, k, n, ir, ig, ib;
  double hmin, hmax, h, dh, v, r, g, b;
  const double s = 1.0;			/* saturation always 1.0 */

  fprintf(stderr, "initial, final hue [°]:\n");
  scanf("%lf%lf", &hmin, &hmax);
  hmin /=  60.0;			/* use multiples of 60° */
  hmax /=  60.0;
  fprintf(stderr, "brightness [0;1]:\n");
  scanf("%lf", &v);
  if (v < 0.0)
    v = 0.0;
  else if (v > 1.0)
    v = 1.0;
  fprintf(stderr, "number of colors:\n");
  scanf("%d", &n);
  if (n <= 0) n = 1;
  dh = (n == 1) ? hmax - hmin : (hmax - hmin) / (n - 1);

  for (i = 0; i < n; i++) {
    h = (hmin + i * dh, 6.0);
    h = hmin + i * dh;
    h = h - 6 * floor(h / 6.0);		/* confine to [0; 6] */
    k = floor(h);
    switch (k) {
      case 0:
        r = v;
	g = v * (1.0 - s * (1.0 - h));
	b = v * (1.0 - s);
	break;
      case 1:
        r = v * (1.0 - s * (h - 1.0));
	g = v;
	b = v * (1.0 - s);
	break;
      case 2:
	r = v * (1.0 - s);
	g = v;
	b = v * (1.0 - s * (3.0 - h));
	break;
      case 3:
	r = v * (1.0 - s);
        g = v * (1.0 - s * (h - 3.0));
	b = v;
        break;
      case 4:
	r = v * (1.0 - s * (5.0 - h));
	g = v * (1.0 - s);
	b = v;
        break;
      default:
	r = v;
	g = v * (1.0 - s);
        b = v * (1.0 - s * (h - 5.0));
    }
    ir = 255 * r;
    ig = 255 * g;
    ib = 255 * b;
    printf("%4d %7.4f %7.4f %7.4f  0x%02x%02x%02x\n", i, r, g, b,
    ir, ig, ib);
  }
}

