/************************************************************************/
/* extract_cols: extract a pair of numerical columns from a file	*/
/************************************************************************/
/* I/O channels:
   stdin	user input
   stdout	extracted data
   stderr	program messages
   cmd line	name of input file
   infile	this file

   stdin sequence:
    1. If the file from which data are to be extracted, has not been
       specified on the command line, enter it now.
    2. Use the following commands to mark file portions for extraction:
       +n	move forward n lines
       -n	move backwards n lines
       i	make the actual line the initial line of the selected
		block of data
       f	make the actual line the final line of the selection
       x   	extract 2 columns and exit.				*/
/************************************************************************/
#include <stdio.h>
#include <string.h>

#define	MAXLINE		256
#define MAXFIELD	50

int main(int argc, char *argv[]) {
  int i, n, nline, ninit, nfinal, xcol, ycol;
  char *p, aux[MAXLINE], line[MAXLINE], cmd[20], x[MAXFIELD], y[MAXFIELD],
       fname[80];
  FILE *infile;

  if (argc == 2)
    strcpy(fname, argv[1]);
  else {
    fprintf(stderr, "input file: ");
    scanf("%s%*c", fname);
  }
  if ((infile = fopen(fname, "r")) == NULL) {
    fprintf(stderr, "extract_cols: could not open file %s\n", fname);
    return (1);
  }

  nline = 0;
  ninit = nfinal = -1;
  strcpy(cmd, "+1");

  while (cmd[0] != 'q') {
    switch (cmd[0]) {
    case '+':				/* scroll forward */
    case '\0':
    case '\n':
      i = sscanf(cmd + 1, "%d", &n);
      if (i < 1) n = 1;
      if (n < 1) break;
      for (i = 0; i < n; i++) {
	strcpy(aux, line);
	p = fgets(line, MAXLINE, infile);
	if (p == '\0') {
	  fprintf(stderr, "end of file!\n");
	  strcpy(line, aux);
	  break;
	}
	else {
	  nline++;
	  fprintf(stderr, "%4d: %s", nline, line);
	}
      }
      break;
    case '-':
      if (nline == 1) 
	fprintf(stderr, "already at begin of file!\n");
      else {
	i = sscanf(cmd + 1, "%d", &n);
	if (i < 1) n = 1;
	if (n < 1) break;
	nline -= n;
	if (nline < 1) nline = 1;
	rewind(infile);
	for (i = 0; i < nline; i++) fgets(line, MAXLINE, infile);
	fprintf(stderr, "%4d: %s", nline, line);
      }
      break;
    case 'i':				/* mark initial line */
      ninit = nline;
      fprintf(stderr, "data block begin set at line %d\n", ninit);
      break;
    case 'f':				/* mark final line */
      nfinal = nline;
      fprintf(stderr, "data block end set at line %d\n", nfinal);
      break;
    case 'x':
      fprintf(stderr, "give the numbers of two colums: ");
      scanf("%d%d", &xcol, &ycol);
      rewind(infile);
      for (i = 1; i < ninit; i++) fgets(line, MAXLINE, infile);
      *aux = '\0';
      if (xcol < ycol) {
	for (i = 1; i < xcol; i++) strcat(aux, "%*s");
	strcat(aux, "%s");
	for (i = xcol + 1; i < ycol; i++) strcat(aux, "%*s");
	strcat(aux, "%s");
      }
      else {
	for (i = 1; i < ycol; i++) strcat(aux, "%*s");
	strcat(aux, "%s");
	for (i = ycol + 1; i < xcol; i++) strcat(aux, "%*s");
	strcat(aux, "%s");
      }
      for (i = ninit; i <= nfinal; i++) {
	fgets(line, MAXLINE, infile);
	if ((p = strchr(line, '\n')) != NULL) *p = '\0';
	if (xcol < ycol)
	  sscanf(line, aux, x, y);
	else
	  sscanf(line, aux, y, x);
	printf("%s %s\n", x, y);
      }
      return (0);
    default:
      fprintf(stderr, "available commands:\n");
      fprintf(stderr, "+[n] move forward n lines\n");
      fprintf(stderr, "-[n] move backward  lines\n");
      fprintf(stderr, "i    set actual line as initial line of data block\n");
      fprintf(stderr, "f    set actual line as end line of data block\n");
      fprintf(stderr, "q    exit without extracting anything\n");
      fprintf(stderr, "x    extract 2 columns of data and exit\n");
    }
    fprintf(stderr, "command: ");
    fgets(cmd, sizeof(cmd), stdin);
  }
  return (0);
}
