/************************************************************************/
/* grx_units: convert coordinates from PostScript to other Grafix units	*/
/************************************************************************/
/* I/O channels:
   stdin	user input
   stdout	converted coordinates
   stderr	error messages
   gfile	global tag file "global.xml"

   stdin sequence:
    1. Enter the x and y coordinates as displayed by Ghostview.		*/
/************************************************************************/
#include <stdio.h>

int main(void) {
  int i, k, n, err;
  double x[2], pt[2], diagr[2][2], graph[2][2], scale[2][2];
  char c, s[20];
  static char *attnames[] = {"diagr-llx", "diagr-lly", "diagr-urx",
    "diagr-ury", "graph-llx", "graph-lly", "graph-urx", "graph-ury",
    "scale-llx", "scale-lly", "scale-urx", "scale-ury"};
  FILE *gfile;

  if ((gfile = fopen("global.xml", "r")) == NULL) {
    fprintf(stderr, "grx_units: cannot open global.xml\n");
    return (1);
  }

  do {					/* look for <cartesian> tag */
    while ((c = getc(gfile)) != '<') {}
    fscanf(gfile, "%s%*c", s);
  } while (strcmp(s, "cartesian"));

  err = 0;
  while (err < ((1 << 12) - 1)) {
    while ((*s = getc(gfile)) <= 0x40) {}	/* skip over white space */
    fscanf(gfile, "%[^=]%*c", s + 1);	/* read an attribute name */
    for (i = 0, n = -1; i < 12; i++) {
      if (!strcmp(s, attnames[i])) {
        n = i;
	break;
      }
    }

    if (n >= 0) {			/* scaling attribute recognized */
      err |= 1 << n;
      k = n % 2;
      n /= 2;
      i = n % 2;
      n /= 2;

      switch (n) {
      case 0:
	fscanf(gfile, "%*c%lf%[^\"]%*c", &diagr[i][k], s);
	if (!strcmp(s, "mm"))
	  diagr[i][k] *= 72.0/25.4;
	else if (!strcmp(s, "cm"))
	  diagr[i][k] *= 72.0/2.54;
	else if (!strcmp(s, "in"))
	  diagr[i][k] *= 72.0;
	break;
      case 1:
	fscanf(gfile, "%*c%lf%*c", &graph[i][k]);
	break;
      case 2:
	fscanf(gfile, "%*c%lf%*[^\"]%*c", &scale[i][k]);
      }
    }
  }

  printf("x, y coordinates (gv display): ");
  while ((i = scanf("%lf%lf", &pt[0], &pt[1])) == 2) {;
    printf("\ncoordinates:\n");
    printf("%12gpt %12gpt\n", pt[0], pt[1]);
    for (k = 0; k < 2; k++) x[k] = pt[k] / 72.0;
    printf("%12gin %12gin\n", x[0], x[1]);
    for (k = 0; k < 2; k++) x[k] = pt[k] * (25.4 / 72.0);
    printf("%12gmm %12gmm\n", x[0], x[1]);
    for (k = 0; k < 2; k++) x[k] = pt[k] * (2.54 / 72.0);
    printf("%12gcm %12gcm\n", x[0], x[1]);
    for (k = 0; k < 2; k++)
      x[k] = (pt[k] - diagr[0][k]) / (diagr[1][0] - diagr[0][0]);
    printf("%12g   %12g  \n", x[0], x[1]);
    for (k = 0; k < 2; k++) x[k] = scale[0][k] + (x[k] - graph[0][k])
    * (scale[1][k] - scale[0][k]) / (graph[1][k] - graph[0][k]);
    printf("%12gxu %12gyu\n", x[0], x[1]);

    printf("x, y coordinates (CTRL-d terminates): ");
  }
  return (0);
}
