/************************************************************************/
/* print_special: print special characters (XML-& encoded characters),	*/
/*		  e.g., accented characters				*/
/************************************************************************/
/* code		(i)  XML character code					*/
/*======================================================================*/
#include <stdio.h>

void print_special(const char *code) {
  int i;
  struct transtl {
    char *xml, *postscript;
  } tt[] =
    {{"aacute",	"\\333"},		/* diacritical marks */
     {"Aacute",	"\\334"},
#if 0
     {"abreve",	"\\u{a}"},
     {"Abreve",	"\\u{A}"},
     {"acedil",	"\\c{a}"},
     {"Acedil",	"\\c{A}"},
     {"acirc",	"\\^a"},
     {"Acirc",	"\\^A"},
#endif
     {"agrave",	"\\355"},
     {"Agrave",	"\\356"},
#if 0
     {"amacr",	"\\=a"},
     {"Amacr",	"\\=A"},
#endif
     {"aring",	"\\260"},
     {"Aring",	"\\265"},
#if 0
     {"atilde",	"\\~a"},
     {"Atilde",	"\\~A"},
#endif
     {"auml",	"\\321"},
     {"Auml",	"\\322"},
#if 0
     {"ccaron",	"\\v{c}"},
     {"Ccaron",	"\\v{C}"},
#endif
     {"ccedil",	"\\276"},
     {"Ccedil",	"\\300"},
     {"eacute",	"\\335"},
     {"Eacute",	"\\336"},
#if 0
     {"ebreve",	"\\u{e}"},
     {"Ebreve",	"\\u{E}"},
     {"ecirc",	"\\^e"},
     {"Ecirc",	"\\^E"},
#endif
     {"egrave",	"\\357"},
     {"Egrave",	"\\360"},
#if 0
     {"emacr",	"\\=e"},
     {"Emacr",	"\\=E"},
#endif
     {"euml",	"\\323"},
     {"Euml",	"\\324"},
     {"iacute",	"\\337"},
     {"Iacute",	"\\340"},
#if 0
     {"ibreve",	"\\u{\\i}"},
     {"Ibreve",	"\\u{\\I}"},
     {"icirc",	"\\^{\\i}"},
     {"Icirc",	"\\^{\\I}"},
     {"igrave",	"\\`{\\i}"},
     {"Igrave",	"\\`{\\I}"},
     {"imacr",	"\\={\\i}"},
     {"Imacr",	"\\={\\I}"},
#endif
     {"iuml",	"\\325"},
     {"Iuml",	"\\326"},
     {"ntilde",	"\\311"},
     {"Ntilde",	"\\314"},
     {"oacute",	"\\342"},
     {"Oacute",	"\\344"},
#if 0
     {"obreve",	"\\u{o}"},
     {"Obreve",	"\\u{O}"},
     {"ocirc",	"\\^o"},
     {"Ocirc",	"\\^O"},
     {"ograve",	"\\`o"},
     {"Ograve",	"\\`O"},
     {"omacr",	"\\=o"},
     {"Omacr",	"\\=O"},
     {"otilde",	"\\~o"},
     {"Otilde",	"\\~O"},
#endif
     {"ouml",	"\\327"},
     {"Ouml",	"\\330"},
#if 0
     {"rcaron",	"\\v{r}"},
     {"Rcaron",	"\\v{R}"},
     {"scaron",	"\\v{s}"},
     {"Scaron",	"\\v{S}"},
     {"tcedil",	"\\c{t}"},
     {"Tcedil",	"\\c{T}"},
#endif
     {"uacute",	"\\345"},
     {"Uacute",	"\\346"},
#if 0
     {"ubreve",	"\\u{u}"},
     {"Ubreve",	"\\u{U}"},
     {"ucirc",	"\\^u"},
     {"Ucirc",	"\\^U"},
     {"ugrave",	"\\`u"},
     {"Ugrave",	"\\`U"},
     {"umacr",	"\\=u"},
     {"Umacr",	"\\=U"},
#endif
     {"uuml",	"\\331"},
     {"Uuml",	"\\332"},
     {"yacute",	"\\347"},
     {"Yacute",	"\\354"},
#if 0
     {"ybreve",	"\\u{y}"},
     {"Ybreve",	"\\u{Y}"},
     {"ycirc",	"\\^y"},
     {"Ycirc",	"\\^Y"},
     {"ygrave",	"\\`y"},
     {"Ygrave",	"\\`Y"},
     {"ymacr",	"\\=y"},
     {"Ymacr",	"\\=Y"},
     {"yuml",	"\\\"y"},
     {"Yuml",	"\\\"Y"},
     {"zcaron",	"\\v{z}"},
     {"Zcaron",	"\\v{Z}"},
#endif
     {"aelig",	"\\361"},		/* ligatures */
     {"AElig",	"\\341"},
     {"oelig",	"\\372"},
     {"OElig",	"\\352"},
     {"oslash",	"\\371"},
     {"Oslash",	"\\351"},
     {"lstrok",	"\\370"},
     {"Lstrok",	"\\350"},
     {"szlig",	"\\373"},
     {"mdash",	"\\320"},		/* interpunction */
     {"ndash",	"\\261"},
     {"hellip",	"\\274"},			/* ellipse */
     {"lsquor",	"\\270"},			/* German single quotes */
     {"lsquo",	"\\140"},
     {"ldquor",	"\\271"},			/* German double quotes */
     {"ldquo",	"\\252"},
     {"slaquo",	"\\254"},			/* French single quotes */
     {"sraquo",	"\\255"},
     {"laquo",	"\\253"},			/* French double quotes */
     {"raquo",	"\\273"},
     {"iquest",	"\\277"},
     {"iexcl",	"\\241"},
     {"amp",	"\\046"},			/* protected characters */
     {"gt",	"\\076"},
     {"lt",	"\\074"},
     {"bull",	"\\267"},		/* special symbols */
     {"sect",	"\\247"},
#if 0
     {"reg",	"\\Pisymbol{psy}{226}"},
     {"copy",	"\\Pisymbol{psy}{227}"},
     {"trade",	"\\Pisymbol{psy}{228}"},
#endif
     {"",	""}
  };

  i = 0;
  while (strcmp(tt[i].xml, "") != 0) { 
    if (strcmp(tt[i].xml, code) == 0) {
      printf("%s", tt[i].postscript);
      return;
    };
    i++;
  }
  printf("??? %s ???", code);
}
